/*
* Define standard functions for the lenstool parameters.
*/

#ifndef PARAMS_H
#define PARAMS_H

/*
 * GLOBAL VARIABLES
 */
 
static int computedr, potparam; // remember the parameters changes before commiting
static int npars;      // number of parameters to optimise
static int ipar;        // current index in the functions lists
static int *lenslist;   // lookup list of lens index associated to the set/etc. functions 

static void **init;     // list of _init() parameters functions
static void **set;      // list of _set() parameters functions
static void **reshape;  // list of _reshape() parameters functions
static void **getName;  // list of _getName() parameters functions
static void **reset;    // list of _reset() parameters functions

/*
 * GLOBAL FUNCTIONS
 */

void initParams();        // Initialise the global variables according to block,cblock...
void appendParamByName(char *name); // Append a new element to functions lists
int  getNparams();        // Return the number of elements in the lists
void commitParams();      // Commit the changes 

void *getInitFunc(int ipx);  // Return the init function pointer for parameter index ipx
void *getSetFunc(int ipx);   // Return the set function pointer for parameter index ipx
void *getReshapeFunc(int ipx);   // Return the reshape function pointer for parameter index ipx
void *getGetNameFunc(int ipx);   // Return the getName function pointer for parameter index ipx
void *getResetFunc(int ipx); // Return the reset function pointer for parameter index ipx 


/* 
 * LOCAL FUNCTIONS 
 * 
 *  
 * INIT SECTION 
 */

// Lens parameters
static void init_cx();
static void init_cy();
static void init_epot();
static void init_theta();
static void init_rc();
static void init_sigma();
static void init_alpha();
static void init_beta();
static void init_rcut();
static void init_masse();
static void init_pmass();

// Cosmo parameters
static void init_omegam();
static void init_omegax();
static void init_wx();

// z_m_limit parameters
static void init_zlim();

// potfile parameters
static void init_rcutStar();
static void init_sigmaStar();

// sigpos & sigell parameters
static void init_sigpos();
static void init_sigell();

/* 
 * SET SECTION 
 */

// Lens parameters
static void set_cx(double val);
static void set_cy(double val);
static void set_epot(double val);
static void set_theta(double val);
static void set_rc(double val);
static void set_sigma(double val);
static void set_alpha(double val);
static void set_beta(double val);
static void set_rcut(double val);
static void set_masse(double val);
static void set_pmass(double val);

// Cosmo parameters
static void set_omegam(double val);
static void set_omegax(double val);
static void set_wx(double val);

// z_m_limit parameters
static void set_zlim(double val);

// potfile parameters
static void set_rcutStar(double val);
static void set_sigmaStar(double val);

// sigpos & sigell parameters
static void set_sigpos(double val);
static void set_sigell(double val);


/* 
 * RESHAPE SECTION 
 */

// Lens parameters
static void reshape_cx(double val);
static void reshape_cy(double val);
static void reshape_epot(double val);
static void reshape_theta(double val);
static void reshape_rc(double val);
static void reshape_sigma(double val);
static void reshape_alpha(double val);
static void reshape_beta(double val);
static void reshape_rcut(double val);
static void reshape_masse(double val);
static void reshape_pmass(double val);

// Cosmo parameters
static void reshape_omegam(double val);
static void reshape_omegax(double val);
static void reshape_wx(double val);

// z_m_limit parameters
static void reshape_zlim(double val);

// potfile parameters
static void reshape_rcutStar(double val);
static void reshape_sigmaStar(double val);

// sigpos & sigell parameters
static void reshape_sigpos(double val);
static void reshape_sigell(double val);

/* 
 * GETNAME SECTION 
 */

// Lens parameters
static void getName_cx();
static void getName_cy();
static void getName_epot();
static void getName_theta();
static void getName_rc();
static void getName_sigma();
static void getName_alpha();
static void getName_beta();
static void getName_rcut();
static void getName_masse();
static void getName_pmass();

// Cosmo parameters
static void getName_omegam();
static void getName_omegax();
static void getName_wx();

// z_m_limit parameters
static void getName_zlim();

// potfile parameters
static void getName_rcutStar();
static void getName_sigmaStar();

// sigpos & sigell parameters
static void getName_sigpos();
static void getName_sigell();


/* 
 * RESET SECTION 
 */

// Lens parameters
static void reset_cx();
static void reset_cy();
static void reset_epot();
static void reset_theta();
static void reset_rc();
static void reset_sigma();
static void reset_alpha();
static void reset_beta();
static void reset_rcut();
static void reset_masse();
static void reset_pmass();

// Cosmo parameters
static void reset_omegam();
static void reset_omegax();
static void reset_wx();

// z_m_limit parameters
static void reset_zlim();

// potfile parameters
static void reset_rcutStar();
static void reset_sigmaStar();

// sigpos & sigell parameters
static void reset_sigpos();
static void reset_sigell();


#endif 
