#ifndef STRUCTURE_H
#define STRUCTURE_H

#include "dimension.h"

// Parameter constants
#define CX       0 
#define CY       1
#define EPOT     2
#define THETA    3
#define RC       4
#define B0       5
#define ALPHA    6 
#define BETA     7
#define RCUT     8
#define MASSE    9
#define PMASS   10
#define OMEGAM  11
#define OMEGAX  12
#define WX      13

/*
* structure definition
*/

/*****************************************************************/
/*								 */
/*		Definition de type 				 */
/*								 */
/*****************************************************************/

typedef struct {
		double re;
		double im;
		} complex;

/*****************************************************************/
/*								 */
/*		structure  point 			 	 */
/*								 */
/*****************************************************************/

struct point {
		double x;
		double y;
		};

/*****************************************************************/
/*								 */
/*		structure  lens-data			 	 */
/*								 */
/*****************************************************************/

typedef struct {
		double alpha_now,x_now,kappa,dpl;
  		} lensdata;


/*****************************************************************/
/*								 */
/*		Definition des structures de controle	 	 */
/*								 */
/*****************************************************************/

struct g_observ {
		int		bruit;
		int		setbin;
		int		bin;
		int		setseeing;
		int		filtre;
		double	seeing;
		double	r0st;
		double	prec;
		double	SKY;
		double	gain;
		int		idum;
		};

struct g_mode {
		int		verbose;
		int		sort;
		int		grille;
		int		ngrille;
		double	zgrille;
		int		inverse;	/* mode inversion de la lentille */
		int		itmax;		/* iter. max lors de l'optimisation*/
		double	rate;		/* rate in the bayesian optimisation*/
		int		image;		/* flag pour fichier d'images */
		char		imafile[50];
		int		source;		/* flag pour fichier de sources */
		char		sourfile[50];
/* from an image file compute only the source catalogue */
		int		sof;		/* flag pour fichier de sources */
		char		imsfile[50];
		char		sfile[50];
/* correlate shear with model */
		int		icorshear;	/* flag pour fichier de correlation */
		char		corshfile[50];
/* etude pour le comportement local des image */
		int		local;	/* flag d'etude de e_i dans le repere local */
		char		localfile[50];
/* etude pour l'inversion des arclets en fonction du redshift */
		int		study;	/* flag d'etude de e_s en fonction de z_s */
		char		studyfile[50];
		int   	fake;
		int   	mean;
		double   	seeing;
/* etude pour le calcul du amplification du plan image */
		int 		iampli;	
		int 		nampli;	
		double 	zampli;
		char		amplifile[50];
/* etude pour le calcul du poten du plan image */
		int 		ipoten;	
		int 		npoten;	
		double 	zpoten;
		char		potenfile[50];
/* etude pour le calcul du mass du plan image */
		int 		imass;	
		int 		nmass;	
		double 	zmass;
		char		massfile[50];
/* etude pour le calcul du dpl du plan image */
		int 		idpl;	
		int 		ndpl;	
		double 	zdpl;
		char		dplxfile[50];
		char		dplyfile[50];
/* etude pour le calcul du curvature du plan image */
		int 		icurv;	
		int 		ncurv;	
		double 	zcurv;
		char		cxxfile[50];
		char		cxyfile[50];
		char		cyyfile[50];
/* etude pour le calcul du shear-field du plan image */
		int 		ishearf;	
		double 	zshearf;
		char		shearffile[50];
		int		nshearf;
/* etude pour le calcul du amplification-field du plan image */
		int 		iamplif;	
		double 	zamplif;
		char		ampliffile[50];
/* etude pour le calcul du shear du plan image */
		int 		ishear;	
		int 		nshear;	
		double 	zshear;
		char		shearfile[50];
/* etude pour le calcul du time_delay du plan image */
		int 		itime;	
		int 		ntime;	
		double 	ztime;
		char		timefile[50];
/* etude pour le calcul des props du plan image */
		int 		prop;	
		int 		nprop;	
		double 	zprop;
		char		propfile[50];
		double	radius;
		double	masse;
/* visualisation pixelise du champ */
		int 		pixel;
		int 		npixel;
		char		pixelfile[50];
/* Reference absolue */
		int     	iref;
		double   	ref_ra;
		double   	ref_dec;
/* calcul des markers sources de markers images donnes */
		int 		marker;
		double 	zmarker;
		char		markfile[50];
/* etude d'une coupe des props du plan image */
		int     	radial;
		double	zradial;
		double   	theta;
/* visualisation pixelise du plan source */
		int		iclean;
		double	zclean;
/* center of multiple images file */
		char	centerfile[50];
/* minimal chi2 for the optimisation*/
		double	minchi0;
		};
		
/* potfile parameters */
struct	g_pot	{
		int		ftype;
		char		potfile[50];
		int		type;
		double	zlens;
		double	core;
		double	corekpc;
		double	mag0;
		int		ircut;
		double	cut,cut1,cut2;
		double	cutkpc1,cutkpc2;
		int		isigma;
		double	sigma,sigma1,sigma2;
		int		islope;
		double	slope,slope1,slope2;
		int		ivdslope;
		double	vdslope,vdslope1,vdslope2;
		};

// parameters of an image in pixels with WCS coordinates
struct g_pixel {
		int		column;
		int		ech;
		int		format;
		char		pixfile[50];
		int		ncont;
		char		outfile[50];
		char		contfile[10][50];
		double	pixelx;
		double	pixely;
		int		header;		
		double	xmin;
		double	xmax;
		double	ymin;
		double	ymax;
		int		nx;
		int		ny;
		struct 	WorldCoor *wcsinfo;
		};

struct g_image {
		int		random;
		int		nzlim;
		int		npcl;
/* shear map parameteres */
		int		shmap;
		double	zsh,drsh;
		char		shfile[50];
/* arclets parameteres */
		int		stat;
		int		statmode;
		char		arclet[50];
		char		nza[10];	/*index of the redshift known arclet in z_arclet*/
		double	zarclet;
		double	drarclet;
		double	sigell,dsigell;	// ellipticity of sources and associated error
		double	sig2ell;
/* multiple arcs parameteres */
        	int		forme;
		int		n_mult;
		int		mult_abs;
		int 		mult[NIMAX];
		char		multfile[50];
		double	sigposAs,dsigposAs;	// images position nuisance and associated error 
		double	sig2pos;
		double	sig2amp;
		double	sig2flux;
		double	Dmag;
		int		adjust;
		char		Afile[50];
		int		Anx;
		int		Any;
		int		Abin;
		int		Anfilt;
		int		Anpixseeing;
		double	Apixel;
		double	Aseeing;
		double	Axmin;
		double	Axmax;
		double	Aymin;
		double	Aymax;
		double	Amean;
		double	Adisp;
		};

struct g_source {
		int		grid;
		int		rand;
		int		ns;
		int		distz;
		double	emax;
		double	zs;
		double	zsmin;
		double	zsmax;
		double	taille;
		double	lfalpha;
		double	lfm_star;
		double	lfm_min;
		double	lfm_max;
		};
		
struct g_grille {
		int		ngrid;
		int		pol;
		int		nlens;
		int		nplens;
		int		nlens_crit;
		int		no_lens;
		char		splinefile[50];
		double	xmin;
		double	xmax;
		double	ymin;
		double	ymax;
		double	dx;
		double	dy;
		int		nx;
		int		ny;
		int		echant;
		double	exc;
		double	excmin;
		};

struct g_cline {
		int		nplan;
		double	cz[NPZMAX];
		int		zone;
		int		npzone;
		char		zonefile[50];
		double	cpas;	// minimum size of the squares in MARCHINGSQUARES or initial step size in SNAKE
		double	dmax;
		char		algorithm[20];	//SNAKE or MARCHINGSQUARES algorithm for critical lines search
		double	limitHigh; // maximum size of the squares in MARCHINGSQUARES algorithm
		};
		
struct g_frame {
		double	xmin;
		double	xmax;
		double	ymin;
		double	ymax;
		double	rmax;
		};
struct g_large {
		double	dlarge;
		char		iname[50];
		int		vitesse;
		int		iso;
		int		nmaxiso;
		double	scale;
		double	zonex;
		double	zoney;
		int		profil;
		int		pt;
		int		ncourbe;
		int		npt;
		};
		
struct g_cosmo {
		double	omegaM;
		double	omegaX;
		double	kcourb;
		double	wX;
		double	H0;
		double	h;
		};
		
/*****************************************************************/
/*								 */
/*      		Definition des types			 */
/*								 */
/*****************************************************************/

struct polar {
		double 	r;
		double 	theta;
		};
		
struct segment {
		double 	dist;
		int 		k;
		};

struct matrix {
		double 	a;
		double 	b;
		double 	c;
		double 	d;
		};
		
struct vecteur {
		double 	x;
		double 	y;
		};

struct biline {
		int 		i;
		struct point I;
		struct point S;
		};
		
struct ellipse {
		double 	a;
		double 	b;
		double 	theta;
		};

struct pointgal {
		int 		n;
		struct point C;
		double 	z;
		};

struct pointell {
		struct point   C;
		struct ellipse E;
		};

struct shear {
		int n;
		struct point C;
		double 	mx;
		double 	my;
		double 	dx;
		double 	dy;
		double 	err;
		};

struct galaxie {
		char 	n[IDSIZE];
		struct point 	C;
		struct point 	Grad;	// total deflection with all clumps
		struct ellipse 	E;
		char  	c;
		int   	type;
		double 	magabs;
		double 	mag;
		double 	flux;
		double 	mu;
		double 	I0;
		double 	z;
		double 	dr;
		double 	q;
		double 	eps;
		double 	tau;
		double 	dis;
		double 	A;
		double 	ep;
		double 	tp;
		double 	dp;
		double 	thp;
		double 	taux;
		double 	tauy;
		double 	time;
		double 	kappa;
		double 	gamma1;
		double 	gamma2;
		double 	deps;
		double 	dtheta;
		struct point grad;		// deflection due to the not optimised clumps
		struct matrix grad2;	// absolute projected potential of the not optimised clumps
		};

struct gal_pol {
		int 	n;
		struct polar 	C;
		struct ellipse 	E;
		double 	z;
		};

struct z_lim {
		int     opt;
		char	n[IDSIZE];
		int		bk;
		int		bk0;
		double   percent;
		double	min;
		double	max;
		double	err;
		double	ddmin;
		double	ddmax;
		double	dderr;
		double	excu;
		double	excd;
		};
		
struct propertie {
		int		n;
		struct point P;
		double 	k;
		double 	s;
		double 	e;
		double 	t;
		double 	q;
		double 	theta;
		double 	d;
		double 	A;
		double 	g;
		};

struct ligne {
		double 	i;
		double 	e;
		double 	theta;
		};

struct line	{
		double 	e;
		double 	theta;
		double 	i;
		double 	phi;
		};

struct cline {
		int		n;
		struct	point	C;
		double	phi;
		double	dl;
		double	z;
		double	dlsds;
		};

struct pot {
		int 		type;
		char 		n[IDSIZE];
		struct point C;
		double 	epot;
		double 	emass;
		double 	theta;
		double 	sigma;
		double 	rc;
		double 	rckpc;
		double 	rcut;
		double 	rcutkpc;
		double 	alpha;
		double 	beta;
		double 	rcslope;
		double 	z;
		double 	dlsds;	/*ratio D(LS)/D(OS) angular distances*/
		double 	psicut;
		double 	psimcut;
		double 	psiccut;
		double 	b0;
		double 	masse;
		double 	pmass;
		double 	cr;
		double 	ct;
		double 	mag;
		double 	lum;
		double 	mtol;
		double 	omegaM;
		double 	omegaX;
		double 	wX;
		};

struct ipot {
		int 	pmax;
		int 	map;
		int 	map_z;
		int 	lens[10];
		int 	para[10];
		int 	lens_z[10];
		int 	para_z[10];
		int 	zlim[2];
		int 	extend;
		int	masse;
		};

struct triplet {
		struct point a;
		struct point b;
		struct point c;
		};

struct bitriplet {
		struct triplet i;
		struct triplet s;
		};

struct chaine {
		struct triplet S;
		struct triplet I;
		struct chaine *F;
		};

struct arbre {
		struct galaxie N;
       	struct arbre *FG;
       	struct arbre *FD;
       	};
       	
struct variation {
		struct point C;
		double elip;
		double theta;
		};

struct pixlist {
		double	i;
		double	j;
		double flux;
		};

struct MCarlo {
		int     optMC;
		int     n_MonteCarlo;
		int     iterations;
		int     squares_par;
		int     tosses_sq;
		};
	        
#endif // if STRUCTURE_H
