#!/usr/bin/perl
# Read all the chires*.dat in the current directory 
# and produce a chires_mean.dat file with the mean
# rms for each system.
# 

if( @ARGV < 1 )
{
	print "Syntax: bayesChires.pl <.par>\n";
	exit -1;
}

$chirestpl="chires{0,1,2,3,4,5,6,7,8,9}*.dat";
$status = system("rm -f $chirestpl");
$status+= system("bayesChires $ARGV[0]");
if( $status != 0 )
{
	exit -1;
}
@chires=`ls $chirestpl`;

# Read the chires*.dat files
for( @chires )
{
	chop;
	open( IN,  $_ );
	$l = 0; # line index in the chires.dat file
	while( <IN> )
	{
		chop;
		@fld = split;
		if( @fld > 8 && $fld[0] ne "N" )
		{
			$id[$l] = $fld[1];
			$z[$l] = $fld[2];
			$narc[$l] = $fld[3]; # always the same number
			$narcs = $narc[$l]; # changing according to chires
			$narc_tot[$l] += $narc[$l]; #total of valid arcs
			$nchires[$l]++; # nb of valid chires
			if( $#fld == 10 ) # nwarn column exists
			{
				$narcs -= $fld[10]; #valid arcs
				$narc_tot[$l] -= $fld[10];
				$nchires[$l]-- if( $narcs == 0 ); #bad chires
			}
			$chi2[$l] += $fld[4]; # total chi2
			$chix[$l] = $fld[5];
			$chiy[$l] = $fld[6];
			$chia[$l] = $fld[7];
			$dx2s[$l] += $fld[8]*$fld[8]*$narcs; #total rmss
			$dx2i[$l] += $fld[9]*$fld[9]*$narcs; #total rmsi
			$l++; # next system
		}
	}
}
close(IN);

# Write the chires_mean.dat file
open(OUT, ">chires_mean.dat");

print OUT "chi multiples\n";
print OUT " N    ID    z   Narcs   chip     chix    chiy    chia  rmss   rmsi\n";
$chi2tot=0;
$dx2stot=0;
$dx2itot=0;
$ntot=0;
for( $i = 0; $i < $l; $i++ )
{
	# $narc_tot : total number of arcs  taken into account for this system
	$rmss = sqrt( $dx2s[$i] / $narc_tot[$i] ) if( $narc_tot[$i] != 0 );
	$rmsi = sqrt( $dx2i[$i] / $narc_tot[$i] ) if( $narc_tot[$i] != 0 );
	$dx2stot += $dx2s[$i];
	$dx2itot += $dx2i[$i];
	$ntot += $narc_tot[$i];
	$chi2[$i]/= $nchires[$i];
	$chi2tot += $chi2[$i];
	printf OUT "%2d %6s %.3f   %d   %7.2f  %6.2f  %6.2f  %6.2f  %.3f  %.2f\n",
		$i, $id[$i], $z[$i], $narc[$i], $chi2[$i],
	       	$chix[$i], $chiy[$i], $chia[$i], $rmss, $rmsi;
}
$rmsstot = $dx2stot / $ntot;
$rmsitot = $dx2itot / $ntot;
printf OUT "chimul                %7.2f  %6.2f  %6.2f  %6.2f  %.3f  %.2f\n",
	$chi2tot, 0, 0, 0, sqrt($rmsstot), sqrt($rmsitot);

close(OUT);
system("rm $chirestpl");
