#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>
#include<lt.h>

/****************************************************************/
/*		nom:		distor				*/
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			*/
/****************************************************************/

/* Write the <dist.dat> file.
 */
static int ampli_error(struct galaxie *image);

void	distor(struct galaxie *image)
{
	extern struct g_mode	M;
	extern struct pot		lens[];
	extern int ni;
	
	int     i;
	double  t0,r,R,E;
	FILE	*OUT;
	struct ellipse amp;
	
	int     errorsOK;
	
	for(i=0;i<ni;i++)
	{
		r=image[i].E.a/image[i].E.b;
		image[i].q=1./r;
		R=r*r;
		E=image[i].eps=(R-1.)/(R+1.);
		image[i].tau=(R-1.)/2./r;
		image[i].time=e_time(image[i].C,image[i].dr);
		amp=e_unmag_gal(&image[i],image[i].dr);
		image[i].A=1/fabs(amp.a*amp.b);
	};
	
	NPRINTF(stderr, "COMP: dist.dat file ");
	errorsOK = ampli_error(image);
	NPRINTF(stderr, "\n");
	
	if (M.sort==1)
		sort(ni,image,comparer_tau);
	else if(M.sort==2)
		sort(ni,image,comparer_pos);	
	
	OUT=fopen("dist.dat","w");
	
	fprintf(OUT,"   #ID       X         Y         R       EPS       TAU       AMP       ");
	if( errorsOK )	fprintf(OUT,"E_AMP      ");
	fprintf(OUT,"DMAG    TIME        DTIME\n");

	t0=image[0].time;
	for(i=0;i<ni;i++)
	{
		fprintf(OUT,"%6s  %8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf  %8.3lf  ",		
			image[i].n,image[i].C.x,image[i].C.y,dist(image[i].C,lens[0].C),
			image[i].eps,image[i].tau,image[i].A);
		if( errorsOK ) fprintf(OUT,"%8.3lf  ",image[i].mu);			
		fprintf(OUT,"%8.2lf  %8.1lf  %8.1lf\n",
			-2.5*log10(1/image[i].A),
			image[i].time,image[i].time-t0);
	};

	fclose(OUT);
}

/* Compute the mean and stddev value of the amplification for
 * a list of images.
 * mean value in image[i].A and stddev value in image[i].mu 
 * 
 * Return >1 if the error have been computed, 0 otherwise.
 */
static int ampli_error(struct galaxie *image)
{
	extern struct   g_mode M;
	extern	int	ni;
	
	struct  ellipse amp;
	double **array;  //contains the bayes.dat file
	double	tmp;
	int		nParam;
	int 	i,nVal, iVal;
	
	array = readBayesModels(&nParam, &nVal);
	
	if( nVal == 0 )
		return 0;
	
	// Set all amplifications and errors to 0
	for( i = 0; i< ni; i++)
		image[i].A = image[i].mu = 0;
	
	for( iVal = 0; iVal < nVal; iVal++ )
	{
		NPRINTF(stderr, "\rCOMP: dist.dat file (Amplification errors %d/%d)",
			iVal+1,nVal);		
		setBayesModel(iVal, nParam, nVal, array);
		for( i = 0; i < ni; i++)
		{
			amp = e_unmag(&image[i].C, image[i].dr );
			tmp = 1./fabs(amp.a*amp.b);
			image[i].A += tmp;
			image[i].mu += tmp*tmp;
		}
	}
	
	// finalize the mean and stddev computation
	for( i = 0; i < ni; i++)
	{
		tmp = image[i].A /= nVal;		// mean
		image[i].mu = (image[i].mu-nVal*tmp*tmp)/(nVal-1);	// variance
		image[i].mu = sqrt(image[i].mu);	// stddev 
	}
	
	NPRINTF(stderr, "\n");
	free(array);
	return nVal;
}
