#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>


static double hypo(double x,double y,int i);

/****************************************************************/
/*		nom:		e_grad				*/
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			*/
/****************************************************************
 * Return the gradient of the projected lens potential.
 * 
 * !!! You have to multiply by dlsds to obtain the true gradient
 * 
 * Global variables used : 
 * - lens, G
 * - in hypo() : lens
 * - in nfwg_dpl() : lens_table
 */
struct point e_grad_pot(struct point *pi, int i)
{
//	extern  struct  g_mode      M;
	extern struct g_grille G;
	extern struct pot      lens[];
	struct pot *ilens;
	double X,Y,R;
	double t,u,q,t05,t10,t15,z;
	double za,zb,zas,zbc,dpl_rad;
	struct point P,Q,Grad,g,pg; //,Qnfw
	struct polar QP;
	complex zis,zis_cut,ztot,ztot_cut;
	
	ilens = &lens[i];
	
	/*positionning at the potential center*/
	P.x=pi->x-ilens->C.x;
	P.y=pi->y-ilens->C.y;
	
	/*rotation to the potential axes*/
	Q=rotation(P,ilens->theta);
	
	switch(ilens->type)
	{
	case(1):
		u=sqrt(hypo(Q.x,Q.y,i));
		g.x=ilens->b0*(1-ilens->epot)*Q.x/u;
		g.y=ilens->b0*(1+ilens->epot)*Q.y/u;
		break;
	case(-1):
		QP=polxy(Q);
		z=sqrt(2.*ilens->epot*3.);
		za=z/sqrt(1+ilens->epot*3.);
		zb=z/sqrt(1-ilens->epot*3.);
		zas=-za*cos(QP.theta);
		zbc=zb*sin(QP.theta);
		pg.x=ilens->b0/z*( -cos(QP.theta)*asin(zas)
			+sin(QP.theta)*asinh(zbc) );
		pg.y=ilens->b0/z*(sin(QP.theta)*asin(zas)
	                +cos(QP.theta)*asinh(zbc) );
		g=rotation(pg,-QP.theta);
		break;
	case(-2):
		zis=csiemd(Q.x,Q.y,ilens->epot,ilens->b0);
		zis_cut=ci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
		g.x =zis.re-zis_cut.re;
		g.y =zis.im-zis_cut.im;
		break;
	
	case(3): // power law with core radius
		z=ilens->rc*ilens->rc;
		t=2*ilens->alpha*ilens->b0/ilens->rc;
		u=pow(1.+(1-ilens->epot)*Q.x*Q.x/z+(1+ilens->epot)*Q.y*Q.y/z,
	            ilens->alpha-1.);
		g.x=t*(1-ilens->epot)*Q.x*u;
		g.y=t*(1+ilens->epot)*Q.y*u;
		break;
	case(4):
		/* ici l'ellipticite est celle du potentiel */
		z=ilens->rc*ilens->rc;
		X=Q.x*Q.x/z; Y=Q.y*Q.y/z;R=X+Y;
		t=ilens->b0/ilens->rc/sqrt(1.+R);
		g.x=t*Q.x*(1-ilens->epot/2./(1.+R)*(2.+X+3.*Y));
		g.y=t*Q.y*(1+ilens->epot/2./(1.+R)*(2.+3.*X+Y));
		break;
	case(41):
		/* same as 4 but with a troncated potentiel
		*/
		z=ilens->rc*ilens->rc;
		X=Q.x*Q.x/z; Y=Q.y*Q.y/z;R=X+Y;
		if (sqrt(R)<ilens->rcut)
		{
			t=ilens->b0/ilens->rc/sqrt(1.+R);
			g.x=t*Q.x*(1-ilens->epot/2./(1.+R)*(2.+X+3.*Y));
			g.y=t*Q.y*(1+ilens->epot/2./(1.+R)*(2.+3.*X+Y));
			/* cut */
			t=2.*ilens->psicut;
			g.x-=t*Q.x;
			g.y-=t*Q.y;
		}
		else
		{
			u=Q.x*Q.x+Q.y*Q.y;
			g.x=ilens->psimcut*Q.x/u;
			g.y=ilens->psimcut*Q.y/u;
		}
		break;
	case(5):
		QP=polxy(Q);
		z=ilens->rc*ilens->rc;
		R=QP.r*QP.r/z;
		t=ilens->b0*ilens->rc/2./QP.r;
		pg.x=t*(log(1.+R)
			-ilens->epot*cos(2.*QP.theta)*(1./(1.+R)-log(1.+R)/R));
		pg.y=ilens->epot*t*sin(2.*QP.theta)*(log(1.+R)/R-1.);
		g=rotation(pg,-QP.theta);
		break;
	case(6):
		QP=polxy(Q);
		z=ilens->rc*ilens->rc;
		R=QP.r*QP.r/z;
		t=ilens->b0/ilens->rc*QP.r;
		pg.x=2.*ilens->alpha*t*pow(1.+R,ilens->alpha-1.)+
			2.*ilens->epot*t*(1.+(1.-ilens->beta)*R)/
			pow(1.+R,1.+ilens->beta)*cos(2.*QP.theta);
		pg.y= -2.*ilens->epot*t/pow(1.+R,ilens->beta)*sin(2.*QP.theta);
		g=rotation(pg,-QP.theta);
		break;
	case(7): /* point mass */
		if (ilens->b0!=0.)
		{
			u=Q.x*Q.x+Q.y*Q.y;
			z=(G.dx*G.dx+G.dy*G.dy)/4.;
       		/* if(u>z) */
       		if ( (fabs(Q.x)>G.dx/2.)||(fabs(Q.y)>G.dy/2.))
			{
				g.x=ilens->b0*Q.x/u;
				g.y=ilens->b0*Q.y/u;
			}
			else
			{
				g.x=ilens->b0*Q.x/z;
				g.y=ilens->b0*Q.y/z;
			}
		}
		else
			g.x=g.y=0.;
        break;
	case(8): /* PIEMD kovner  I0.5 */
		zis=ci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		g.x=zis.re;
		g.y=zis.im;
        break;
	
	case(81): /* PIEMD Kassiola & Kovner,1993 I0.5c-I0.5cut */
	
		t05=ilens->rcut/(ilens->rcut-ilens->rc);
		zis=ci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		zis_cut=ci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
		g.x=t05*(zis.re-zis_cut.re);
		g.y=t05*(zis.im-zis_cut.im);
        break;
	
	case(82): /* PIEMD kovner with shallow central slope I0.5c+I0.5cut*/
	
		t05=(ilens->rcut+ilens->rc)/ilens->rcut;
		zis=ci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		zis_cut=ci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
		g.x=t05*(zis.re+zis_cut.re);
		g.y=t05*(zis.im+zis_cut.im);
        break;
	
	case(83): /* EMD kovner 3/2: I1.5c*/
	
		ztot=ci15(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		g.x=ztot.re;
		g.y=ztot.im;
        break;
	
	case(84): /* EMD kovner isotherme I0.5c-I0.5cut + I1.5c */
	
		t05=ilens->rcut/(ilens->rcut-ilens->rc);
		t15=ilens->rcut/ilens->rc;
		zis=ci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		zis_cut=ci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
        	ztot=ci15(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		q=ilens->alpha;
		g.x=q*t15*ztot.re+ (1-q)*t05*(zis.re-zis_cut.re);
		g.y=q*t15*ztot.im+ (1-q)*t05*(zis.im-zis_cut.im);
        break;
	
	case(85): /* EMD kovner 1: I1c*/
	
		ztot=ci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		g.x=ztot.re;
		g.y=ztot.im;
        break;
	
	case(86): /* EMD kovner 1: I1c-I1cut*/
	
		t10=ilens->rc/(1.-ilens->rc*ilens->rc/ilens->rcut/ilens->rcut);
		ztot=ci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		ztot_cut=ci10(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
		g.x=t10*(ztot.re-ztot_cut.re);
		g.y=t10*(ztot.im-ztot_cut.im);
        break;
	
	case(87): /* EMD kovner  I1c-I1cut +I0.5c-I0.5cut */ 
		
		t05=1./(1.-ilens->rc/ilens->rcut);
		t10=ilens->rc/(1.-ilens->rc*ilens->rc/ilens->rcut/ilens->rcut);
		zis=ci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		zis_cut=ci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
		ztot=ci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		ztot_cut=ci10(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
		q=ilens->alpha;
		g.x=(1-q)*t05*(zis.re-zis_cut.re) +q*t10*(ztot.re-ztot_cut.re);
		g.y=(1-q)*t05*(zis.im-zis_cut.im) +q*t10*(ztot.im-ztot_cut.im);
        break;
	
	case(88): /* EMD kovner  I1c-I1cut +I1.5cut */ 
		
		t10=ilens->rc/(1.-ilens->rc*ilens->rc/ilens->rcut/ilens->rcut);
		t15=ilens->rcut/ilens->rc;
		zis_cut=ci15(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
		ztot=ci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		ztot_cut=ci10(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
		q=ilens->alpha;
		g.x=(1-q)*t15*zis_cut.re +q*t10*(ztot.re-ztot_cut.re);
		g.y=(1-q)*t15*zis_cut.im +q*t10*(ztot.im-ztot_cut.im);
        break;
	
	case(89): /* EMD kovner  I1c-I1cut +I0.5c-I0.5cut */ 
		
		t05=1./(1.-1./ilens->beta);
		t10=ilens->rc/(1.-1./ilens->beta/ilens->beta);
		zis=ci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		zis_cut=ci05(Q.x,Q.y,ilens->epot,ilens->rc*ilens->beta,ilens->b0);
		ztot=ci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
		ztot_cut=ci10(Q.x,Q.y,ilens->epot,ilens->rc*ilens->beta,ilens->b0);
		q=ilens->alpha;
		g.x=q*t05*(zis.re-zis_cut.re) +(1-q)*t10*(ztot.re-ztot_cut.re);
		g.y=q*t05*(zis.im-zis_cut.im) +(1-q)*t10*(ztot.im-ztot_cut.im);
        break;
	
	
	case(9): /* plan masse */
		t=ilens->b0;
		g.x=t*Q.x;
		g.y=t*Q.y;
        break;
	
	case(10): /* spline */
		Grad=sp_grad(*pi);
        break;
	
	case(11): /* 1/r4 mass */
		z=ilens->rc*ilens->rc;
		t=2.*ilens->b0/ilens->rc;
		R=1.+(Q.x*Q.x+Q.y*Q.y)/z;
		g.x=t*Q.x/R;
		g.y=t*Q.y/R;
        break;
	
	case(12): /* NFW */
	  /*     Qnfw.x=Q.x-ilens->rc*ilens->emass/(1.-ilens->emass);
		u=sqrt(Qnfw.x*Qnfw.x+Q.y*Q.y);
		g.x=nfw_dpl(u,ilens->rc,ilens->b0/2.)*Qnfw.x/u;
		g.y=nfw_dpl(u,ilens->rc,ilens->b0/2.)*Q.y/u;
	        Qnfw.x=Q.x+ilens->rc*ilens->emass/(1.-ilens->emass);
		u=sqrt(Qnfw.x*Qnfw.x+Q.y*Q.y);
		g.x+=nfw_dpl(u,ilens->rc,ilens->b0/2.)*Qnfw.x/u;
		g.y+=nfw_dpl(u,ilens->rc,ilens->b0/2.)*Q.y/u;
	   */
	   /*    
		u=sqrt(Q.x*Q.x+Q.y*Q.y);
		g.x=nfw_dpl(u,ilens->rc,ilens->b0)*Q.x/u;
		g.y=nfw_dpl(u,ilens->rc,ilens->b0)*Q.y/u;
	   */
		u = sqrt( (1.-ilens->emass)*Q.x*Q.x+(1.+ilens->emass)*Q.y*Q.y );
		if( u == 0. )
		{
			g.x = g.y = 0.;
			return g;
		}	
		if (ilens->alpha == 0.)
		{
			dpl_rad=nfw_dpl(u,ilens->rc,ilens->b0);	// circular potential
			g.x=dpl_rad*(1.-ilens->emass)*Q.x/u;		// corrections to elliptical potential
			g.y=dpl_rad*(1.+ilens->emass)*Q.y/u;
		}
		else
		{
			dpl_rad=nfwg_dpl(u,ilens->rc,ilens->b0,ilens->alpha);
			g.x=dpl_rad*(1.-ilens->emass)*Q.x/u;
			g.y=dpl_rad*(1.+ilens->emass)*Q.y/u;
		}
		break;	
	case(13): // Sersic
		u = sqrt( (1.-ilens->emass)*Q.x*Q.x+(1.+ilens->emass)*Q.y*Q.y );
		dpl_rad=sersic_dpl(u,ilens->rc,ilens->alpha,ilens->b0);	// circular potential
		g.x=dpl_rad*(1.-ilens->emass)*Q.x/u;		// corrections to elliptical potential
		g.y=dpl_rad*(1.+ilens->emass)*Q.y/u;
		break;
	case(14): // local shear
		g.x=ilens->epot*Q.x;
		g.y=-ilens->epot*Q.y;
		break;
	default:
		fprintf(stderr,"ERROR: profil type of clump %s unknown : %d\n",
			ilens->n, ilens->type);
		exit(-1);
		break;
	};


	if (ilens->type!=10)
		Grad=rotation(g,-ilens->theta);

	return Grad;
}

struct point e_grad(struct point *pi)
{
	extern struct g_grille G;
	struct point Grad, grad;
	int i;
	
	/* NPRINTF(stderr,"e_grad\n"); */
	
	/* here elipticity is the potential one */
	
	grad.x=0.;grad.y=0.;

	/* for each lens*/
	for(i=0;i<G.nlens;i++)
	{
		Grad = e_grad_pot(pi, i);
		grad.x+=Grad.x;
		grad.y+=Grad.y;
	};	/*end of for each lens*/
	
	return (grad);	
}

struct point e_grad_gal(struct galaxie *image)
{
	extern	struct	g_grille	G;
	struct point grad, Grad;
	int i;
	
	grad.x = grad.y = 0;
	
	// conditionaly, get the potential of the not optimised clumps	
	if( image->grad.x == image->grad.y )
	{
		// get the potential of the not optimised clumps
		for( i = G.no_lens; i < G.nlens; i++ )
		{
			Grad = e_grad_pot(&image->C, i);
			grad.x += Grad.x;
			grad.y += Grad.y;
		}
		
		// save the potential of the not optimised clumps
		image->grad.x = grad.x;
		image->grad.y = grad.y;
		
		// get the potential of the optimised clumps
		for( i = 0; i < G.no_lens; i++ )
		{
			Grad = e_grad_pot(&image->C, i);
			grad.x += Grad.x;
			grad.y += Grad.y;
		}		
	} else
	{
		// get the potential of the optimised clumps
		for( i = 0; i < G.no_lens; i++ )
		{
			Grad = e_grad_pot(&image->C, i);
			grad.x += Grad.x;
			grad.y += Grad.y;
		}
		
		// add the potential of the not optimised clumps
		grad.x += image->grad.x;
		grad.y += image->grad.y;
	}
	
	return grad;
}

/***********************************************************
 * Global variables used :
 * - lens
 */
static double hypo(double x,double y,int i)
{
	extern struct pot lens[];
	return( (1-lens[i].epot)*x*x + (1+lens[i].epot)*y*y );
}
