#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>

#define	cube(A)	A*A*A

static double par1(double x, double y, int i);
static double par2(double x, double y, int i);

/****************************************************************/
/*		nom:		e_grad2				*/
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			*/
/****************************************************************
 * Return the projected lenses potential computed in pi. It is
 * the sum over all lenses of their individual potential second 
 * derivatives computed in pi.
 * 
 * This function doesnt call functions that use and modify global variables.
 * 
 * Parameters : 
 * - pi : position of computation in the image plane 
 * 
 * Global variables used :
 * - G, lens
 * - in par1() : lens
 * - in par2() : lens
 * - in ngwg_kappa() : lens_table
 * - in nfwg_gamma() : lens_table
 * - in nfwg_kappa_eps() : lens_table
 * - in nfwg_gamma_eps() : lens_table
 */
struct	matrix	e_grad2(struct point *pi)
{
//	extern  struct  g_mode      M;
	extern	struct	g_grille	G;

	struct matrix  MA, grad2;
	int	i;

	MA.a=MA.b=MA.c=MA.d=0.;
	
	/*for each lens*/
	for(i=0;i<G.nlens;i++)
	{
		grad2 = e_grad2_pot(pi, i);
		MA.a+=grad2.a;
		MA.b+=grad2.b;
		MA.c+=grad2.c;
	};
	
	MA.d=MA.b;
	
	return (MA);
}

/* Special e_grad2() function for the galaxie structures. To use
 * during the optimisation process to spare computation time with 
 * the not optimised clumps.
 */
struct matrix e_grad2_gal(struct galaxie *image)
{
	extern	struct	g_grille	G;
	struct matrix MA, grad2;
	int i;

	MA.a = MA.b = MA.c = MA.d = 0;
	
	// conditionaly, get the potential of the not optimised clumps
	if( image->grad2.a == image->grad2.c )
	{
		// get the potential of the not optimised clumps
		for( i = G.no_lens; i < G.nlens; i++ )
		{
			grad2 = e_grad2_pot(&image->C, i);
			MA.a+=grad2.a;
			MA.b+=grad2.b;
			MA.c+=grad2.c;
		}
		// save the potential of the not optimised clumps
		image->grad2.a = MA.a;
		image->grad2.b = MA.b;
		image->grad2.c = MA.c;		
		
		// get the potential of the optimised clumps
		for( i = 0; i < G.no_lens; i++ )
		{
			grad2 = e_grad2_pot(&image->C, i);
			MA.a+=grad2.a;
			MA.b+=grad2.b;
			MA.c+=grad2.c;
		}
	} else
	{
		// get the potential of the optimised clumps
		for( i = 0; i < G.no_lens; i++ )
		{
			grad2 = e_grad2_pot(&image->C, i);
			MA.a+=grad2.a;
			MA.b+=grad2.b;
			MA.c+=grad2.c;
		}

		// add the potential of the not optimised clumps
		MA.a += image->grad2.a;
		MA.b += image->grad2.b;
		MA.c += image->grad2.c;
	}
	
	MA.d=MA.b;
	
	return (MA);
	
}


struct matrix e_grad2_pot(struct point *pi, int i)
{
	extern struct g_grille	G;
	extern struct pot		lens[];
	struct pot     *ilens;
	struct point   R;  /*position of a lens relative to pi*/
	struct point   Q;  /*position of R rotated so that the lens is // to Xaxis*/
	double ep,em,ee;   /*special potential ellipticities*/
	double t05,t10,t15,q,z,p,al,be;	//potential temporary variables
	double X,Y,RR;
	struct polar   QP;	// for -1 potential
	struct matrix  grad2,g2,g05c,g05cut,g10c,g10cut,g15c,g15cut; //,g21,g20
	struct matrix  gsiemd;
	struct point   Qe;	// elliptical radius NFW/Sersic
	
//	double   za,zb,zas,zbc;
	double  kap,gam;

	ilens = &lens[i];
		
	R.x=pi->x-ilens->C.x;
	R.y=pi->y-ilens->C.y;
	Q=rotation(R,ilens->theta);
	ep=1.+ilens->epot;	/* a/(a+b) */
	em=1.-ilens->epot; /* b/(a+b) */
	ee=1.-ilens->epot*ilens->epot; /* 2ab/(a+b) */

	switch(ilens->type)
	{
		case(1):
			z=ilens->b0*pow(par1(Q.x,Q.y,i),-1.5);
			g2.a=z*ee*Q.y*Q.y;
			g2.b=g2.d= -z*ee*Q.x*Q.y;
			g2.c=z*ee*Q.x*Q.x;
			break;
		case(-1):
			QP=polxy(Q);
			g2.a=0.;
			g2.b=g2.d=0.;
			g2.c=ilens->b0/QP.r/sqrt(1.-ilens->epot*3.*cos(2.*QP.theta));
			g2=rotmatrix(&g2,QP.theta);
			break;
		case(-2):
			mdcsiemd(Q.x,Q.y,ilens->epot,ilens->b0,&gsiemd);
			mdci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0,&g05cut);
		
			g2.a =gsiemd.a-g05cut.a;
			g2.b =gsiemd.b-g05cut.b;
			g2.c =gsiemd.c-g05cut.c;
			g2.d =gsiemd.d-g05cut.d;

			break;
		case(3): // PL with core
			z=ilens->rc*ilens->rc;
			p=par2(Q.x,Q.y,i);
			q=2*ilens->alpha*ilens->b0/ilens->rc*pow(p,ilens->alpha-2.);
			g2.a=em*q*(p+2.*em*(ilens->alpha-1.)*Q.x*Q.x/z);
			g2.b=g2.d=2.*ee*q*(ilens->alpha-1.)*Q.x*Q.y/z;
			g2.c=ep*q*(p+2.*ep*(ilens->alpha-1.)*Q.y*Q.y/z);
			break;
		case(4):
			z=ilens->rc*ilens->rc;
			X=Q.x*Q.x/z; Y=Q.y*Q.y/z; RR=X+Y;
			q=ilens->b0/ilens->rc/pow(1.+RR,1.5);
			g2.a=q*(1.+Y-ilens->epot/2./(1.+RR)*(2.-X+5.*Y-3.*Y*(X-Y)));
			g2.b=g2.d= -q*Q.x*Q.y/z*(1.+ilens->epot*3./2.*(X-Y)/(1.+RR));
			g2.c=q*(1.+X+ilens->epot/2./(1.+RR)*(2.+5.*X-Y+3.*X*(X-Y)));
			break;
		case(41):
			/* same as 4 but with a cut
			*/
			z=ilens->rc*ilens->rc;
			X=Q.x*Q.x/z; Y=Q.y*Q.y/z; RR=X+Y;
			if (sqrt(RR)<ilens->rcut)
			{
				q=ilens->b0/ilens->rc/pow(1.+RR,1.5);
				g2.a=q*(1.+Y-ilens->epot/2./(1.+RR)*(2.-X+5.*Y-3.*Y*(X-Y)));
	       		g2.b=g2.d= -q*Q.x*Q.y/z*(1.+ilens->epot*3./2.*(X-Y)/(1.+RR));
	       		g2.c=q*(1.+X+ilens->epot/2./(1.+RR)*(2.+5.*X-Y+3.*X*(X-Y)));
		
				z=2.*ilens->psicut;
				g2.a-=z;
				g2.c-=z;
				break;
			}
			else
			{
				X=Q.x*Q.x; Y=Q.y*Q.y;
				RR=X+Y;
				z=ilens->psimcut/RR/RR;
				g2.a=z*(Y-X);
				g2.b=g2.d= -2.*z*Q.x*Q.y;
				g2.c=z*(X-Y);
			}
			break;
		case(5):	// not corrected for ellipticity
			QP=polxy(Q);
			z=ilens->rc*ilens->rc;
			RR=QP.r*QP.r/z;
			q=ilens->b0/RR/ilens->rc;
			g2.c=q*(log(1.+RR)/2.+ilens->epot*cos(2.*QP.theta)
				*(log(1.+RR)/RR-(2.+RR)/(1.+RR)));
			g2.b=g2.d=q*ilens->epot*sin(2.*QP.theta)/2.
				*((3.+RR)/(1.+RR)-3.*log(1.+RR)/RR);
			g2.a=q*( (RR/(1.+RR)-log(1.+RR)/2.)
				+ilens->epot/2.*cos(2.*QP.theta)
		    	*(3.*log(1.+RR)/RR-(3.+5.*RR)/(1.+RR)/(1.+RR)));
			g2=rotmatrix(&g2,QP.theta);
			break;
		case(6):	// not corrected for ellipticity
			QP=polxy(Q);
			z=ilens->rc*ilens->rc;
			RR=QP.r*QP.r/z;
			q=2.*ilens->b0/ilens->rc;
			al=ilens->alpha; be=ilens->beta;
			g2.a=al*q*(1.+(2.*al-1.)*RR)/pow(1.+RR,2.-al)+ilens->epot*q
				*(1.+(2.-5.*be)*RR+(2.*be*be-3.*be+1.)*RR*RR)/pow(1.+RR,be+2.)
				*cos(2.*QP.theta);
			g2.b=g2.d= -q*ilens->epot*sin(2.*QP.theta)
				*(1.+(1.-2.*be)*RR)/pow(1.+RR,be+1.);
			g2.c=al*q/pow(1.+RR,1.-al)-ilens->epot*q*cos(2.*QP.theta)
				*(1.+(1.+be)*RR)/pow(1.+RR,be+1.);
			g2=rotmatrix(&g2,QP.theta);
			break;
		case(7):	// not corrected for ellipticity
			if (ilens->b0!=0.)
			{
				X=Q.x*Q.x; Y=Q.y*Q.y;
				RR=X+Y;
				q=(G.dx*G.dx+G.dy*G.dy)/4.;
		                /* if(RR>q) */
				if ((fabs(Q.x)>G.dx/2.)||(fabs(Q.y)>G.dy/2.))
				{
					z=ilens->b0/RR/RR;
					g2.a=z*(Y-X);
					g2.b=g2.d= -2.*z*Q.x*Q.y;
					g2.c=z*(X-Y);
				}
				else
				{
					z=ilens->b0/q;
					g2.a=z;
			        g2.b=g2.d=0.;
		            g2.c=z;
				}
			}
			else
				g2.a=g2.b=g2.d=g2.c=0.;
			break;
		
		case(8): /* PIEMD kovner */
			mdci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0,&g2);
	        break;
		
		case(81): /* trucated PIEMD kovner (HK model1) */
			t05=ilens->rcut/(ilens->rcut-ilens->rc);
			mdci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0,&g05c);
			mdci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0,&g05cut);
			g2.a=t05*(g05c.a-g05cut.a);
			g2.b=t05*(g05c.b-g05cut.b);
			g2.c=t05*(g05c.c-g05cut.c);
			g2.d=t05*(g05c.d-g05cut.d);
	        break;

		case(82): /* PIEMD kovner with a shallower central slope*/
			t05=(ilens->rcut+ilens->rc)/ilens->rcut;
			mdci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0,&g05c);
			mdci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0,&g05cut);
			g2.a=t05*(g05c.a+g05cut.a);
			g2.b=t05*(g05c.b+g05cut.b);
			g2.c=t05*(g05c.c+g05cut.c);
			g2.d=t05*(g05c.d+g05cut.d);
		break;
		
		case(83): /* EMD kovner 3/2 */
			g2=mdci15(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
	        break;
		
		case(84): /* EMD kovner I0.5c-I0.5cut + I1.5c */
		
			t05=ilens->rcut/(ilens->rcut-ilens->rc);
			t15=ilens->rcut/ilens->rc;
			g15c=mdci15(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
			mdci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0,&g05c);
	        	mdci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0,&g05cut);
		
			q=ilens->alpha;
			g2.a=q*t15*g15c.a+(1-q)*t05*(g05c.a-g05cut.a);
			g2.b=q*t15*g15c.b+(1-q)*t05*(g05c.b-g05cut.b);
			g2.c=q*t15*g15c.c+(1-q)*t05*(g05c.c-g05cut.c);
			g2.d=q*t15*g15c.d+(1-q)*t05*(g05c.d-g05cut.d);
	        break;
	
		case(85): /* EMD kovner 1: I1c*/
		
			g2=mdci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
	        break;
		
		case(86): /* EMD kovner 1: I1c-I1cut*/
		
			t10=ilens->rc/(1.-ilens->rc*ilens->rc/ilens->rcut/ilens->rcut);
			g10c=mdci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
			g10cut=mdci10(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
			g2.a=t10*(g10c.a-g10cut.a);
			g2.b=t10*(g10c.b-g10cut.b);
			g2.c=t10*(g10c.c-g10cut.c);
			g2.d=t10*(g10c.d-g10cut.d);
		    break;
		
		case(87): /* EMD kovner 1: I1c-I1cut + I0.5c - I0.5cut */
		
			t05=ilens->rcut/(ilens->rcut-ilens->rc);
			t10=ilens->rc/(1.-ilens->rc*ilens->rc/ilens->rcut/ilens->rcut);
			mdci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0,&g05c);
			mdci05(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0,&g05cut);
			g10c=mdci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
			g10cut=mdci10(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
			q=ilens->alpha;
			g2.a=(1-q)*t05*(g05c.a-g05cut.a)+q*t10*(g10c.a-g10cut.a);
			g2.b=(1-q)*t05*(g05c.b-g05cut.b)+q*t10*(g10c.b-g10cut.b);
			g2.c=(1-q)*t05*(g05c.c-g05cut.c)+q*t10*(g10c.c-g10cut.c);
			g2.d=(1-q)*t05*(g05c.d-g05cut.d)+q*t10*(g10c.d-g10cut.d);
		break;
		
		case(88): /* EMD kovner 1: I1c-I1cut + I1.5cut */
		
			t10=ilens->rc/(1.-ilens->rc*ilens->rc/ilens->rcut/ilens->rcut);
			t15=ilens->rcut/ilens->rc;
			g10c=mdci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
			g10cut=mdci10(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
			g15cut=mdci15(Q.x,Q.y,ilens->epot,ilens->rcut,ilens->b0);
			q=ilens->alpha;
			g2.a=(1-q)*t15*g15cut.a+q*t10*(g10c.a-g10cut.a);
			g2.b=(1-q)*t15*g15cut.b+q*t10*(g10c.b-g10cut.b);
			g2.c=(1-q)*t15*g15cut.c+q*t10*(g10c.c-g10cut.c);
			g2.d=(1-q)*t15*g15cut.d+q*t10*(g10c.d-g10cut.d);
	        break;
		
		case(89): /* EMD kovner 1: I1c-I1cut + I0.5c - I0.5cut */
		
			t05=1./(1.-1./ilens->beta);
			t10=ilens->rc/(1.-1./ilens->beta/ilens->beta);
			mdci05(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0,&g05c);
			mdci05(Q.x,Q.y,ilens->epot,ilens->rc*ilens->beta,ilens->b0,&g05cut);
			g10c=mdci10(Q.x,Q.y,ilens->epot,ilens->rc,ilens->b0);
			g10cut=mdci10(Q.x,Q.y,ilens->epot,ilens->rc*ilens->beta,ilens->b0);
			q=ilens->alpha;
			g2.a=q*t05*(g05c.a-g05cut.a)+(1.-q)*t10*(g10c.a-g10cut.a);
			g2.b=q*t05*(g05c.b-g05cut.b)+(1.-q)*t10*(g10c.b-g10cut.b);
			g2.c=q*t05*(g05c.c-g05cut.c)+(1.-q)*t10*(g10c.c-g10cut.c);
			g2.d=q*t05*(g05c.d-g05cut.d)+(1.-q)*t10*(g10c.d-g10cut.d);
	        break;
		
		case(9):
			g2.a=ilens->b0;
			g2.b=g2.d= 0.;
			g2.c=ilens->b0;
			break;
		
		case(10):
			grad2=sp_grad2(*pi);
			break;
		
		case(11): /* 1/r4 mass */
			z=ilens->rc*ilens->rc;
			q=2.*ilens->b0/ilens->rc;
			X=Q.x*Q.x/z;
			Y=Q.y*Q.y/z;
			RR=1.+(Q.x*Q.x+Q.y*Q.y)/z;
			g2.a=q*(1.-X+Y)/RR;
			g2.b=-2.*q*Q.x*Q.y/z/RR;
			g2.c=q*(1.+X-Y)/RR;
	        break;
		
		case(12): /* NFW */
		  /*  Qe.x=Q.x-ilens->rc*ilens->emass/(1.-ilens->emass);
			Qe.y=Q.y;
			QP=polxy(Qe);
			kap=nfw_kappa(QP.r,ilens->rc,ilens->b0/2.);
			gam=nfw_gamma(QP.r,ilens->rc,ilens->b0/2.);
			g20.a=kap-gam;
			g20.b=g20.d=0.;
			g20.c=kap+gam;
			g20=rotmatrix(g20,QP.theta);
			Qe.x=Q.x+ilens->rc*ilens->emass/(1.-ilens->emass);
			Qe.y=Q.y;
			QP=polxy(Qe);
			kap=nfw_kappa(QP.r,ilens->rc,ilens->b0/2.);
			gam=nfw_gamma(QP.r,ilens->rc,ilens->b0/2.);
			g21.a=kap-gam;
			g21.b=g21.d=0.;
			g21.c=kap+gam;
			g21=rotmatrix(g21,QP.theta);
			g2.a=g21.a+g20.a;
			g2.b=g21.b+g20.b;
			g2.c=g21.c+g20.c;
			g2.d=g21.d+g20.d;
		    */
		    /*
			QP=polxy(Q);
			kap=nfw_kappa(QP.r,ilens->rc,ilens->b0);
			gam=nfw_gamma(QP.r,ilens->rc,ilens->b0);
			g2.a=kap-gam;
			g2.b=g2.d=0.;
			g2.c=kap+gam;
			g2=rotmatrix(g2,QP.theta);
		    */
			Qe.x=sqrt(1.-ilens->emass)*Q.x;
			Qe.y=sqrt(1.+ilens->emass)*Q.y;
			QP=polxy(Qe);
			if (ilens->alpha == 0.)
			{
				kap=nfw_kappa_eps(QP.r,ilens->rc,QP.theta,ilens->b0,ilens->emass);
				gam=nfw_gamma_eps(QP.r,ilens->rc,QP.theta,ilens->b0,ilens->emass);
			}
			else
			{
				kap=nfwg_kappa_eps(QP.r,ilens->rc,QP.theta,ilens->b0,ilens->emass,
					ilens->alpha);
				gam=nfwg_gamma_eps(QP.r,ilens->rc,QP.theta,ilens->b0,ilens->emass,
					ilens->alpha);
			}
			/*	
			kap=nfw_kappa(QP.r,ilens->rc,ilens->b0);
			gam=nfw_gamma(QP.r,ilens->rc,ilens->b0);
			*/
			g2.a=kap-gam;
			g2.b=g2.d=0.;
			g2.c=kap+gam;
			QP=polxy(Q);
			g2=rotmatrix(&g2,QP.theta);
			break;	
		case(13): // Sersic
			Qe.x=sqrt(1.-ilens->emass)*Q.x;
			Qe.y=sqrt(1.+ilens->emass)*Q.y;
			QP=polxy(Qe);
			kap=sersic_kappa_eps(QP.r,ilens->rc,ilens->alpha,QP.theta,ilens->b0,ilens->emass);
			gam=sersic_gamma_eps(QP.r,ilens->rc,ilens->alpha,QP.theta,ilens->b0,ilens->emass);
			g2.a=kap-gam;
			g2.b=g2.d=0.;
			g2.c=kap+gam;
			QP=polxy(Q);
			g2=rotmatrix(&g2,QP.theta);
			break;
		case(14):  // local shear
			g2.a=ilens->epot;
			g2.c=-ilens->epot;
			g2.b=g2.d=0;
			break;
		default:
			fprintf(stderr,"ERROR: profil type of clump %s unknown: %d \n",
				ilens->n, ilens->type);
			exit(-1);
			break;
	};

	if(ilens->type!=10)
		grad2=rotmatrix(&g2,ilens->theta);
		
	return grad2;		
}


/***********************************************************/

static double par1(double x, double y, int i)
{
	extern struct pot lens[];
	return( (1.-lens[i].epot)*x*x + (1.+lens[i].epot)*y*y );
}

/***********************************************************/

static double par2(double x, double y, int i)
{
	extern struct pot lens[];
	double z;
	z=lens[i].rc*lens[i].rc;
	return( 1. + (1-lens[i].epot)*x*x/z + (1+lens[i].epot)*y*y/z );
}
