#include<stdio.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>

/****************************************************************/
/*		nom:		e_im_prec       		*/
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse		
 * 
 * Return in *A the smallest couple of triangles in which P is located according
 * to the minimum surface required for the source triangle.
 * 
 * it global variable is modified. it counts the number of loop needed 
 * to reach the smallest triangle.
 * 
 * Parameters :
 * - E : Triplet of simulated images and corresponding sources (6 points)
 * - P : Barycenter of the original sources
 * - dlsds : Lens efficiency
 * - prec : minimum surface of the smallest source triangle
 * - nitermax : maximum number of loop allowed to reach the smallest triangle
 * 
 * Global variables used :
 * - it
 * - in e_inthere() : G, lens, lens_table
 * - in e_transform() : G, lens, lens_table
 */
void e_im_prec(struct bitriplet *E,	/* Triplet of simulated images and corresponding sources*/
               struct point *P,		/* Barycenter of the original sources */
               double dlsds,double prec,
               int nitermax, struct bitriplet *A)

{
	extern int it;
	
	struct bitriplet I;
	double det;
	
#ifdef DEBUG
	// plot the E triangle in red in the image plane 
	// and in blue in the source plane
	extern struct g_mode M;
	FILE  *dbg;
	
	dbg = fopen( "imsearch.reg", "a");
	fprintf( dbg, "fk5;polygon(%lf,%lf,%lf,%lf,%lf,%lf) #color=red\n",
		M.ref_ra + E->i.a.x /-3600/cos(M.ref_dec*DTR),
		M.ref_dec + E->i.a.y / 3600,
		M.ref_ra + E->i.b.x /-3600/cos(M.ref_dec*DTR),
		M.ref_dec + E->i.b.y / 3600,
		M.ref_ra + E->i.c.x /-3600/cos(M.ref_dec*DTR),
		M.ref_dec + E->i.c.y / 3600 );	
	fprintf( dbg, "fk5;polygon(%lf,%lf,%lf,%lf,%lf,%lf) #color=blue\n",
		M.ref_ra + E->s.a.x /-3600/cos(M.ref_dec*DTR),
		M.ref_dec + E->s.a.y / 3600,
		M.ref_ra + E->s.b.x /-3600/cos(M.ref_dec*DTR),
		M.ref_dec + E->s.b.y / 3600,
		M.ref_ra + E->s.c.x /-3600/cos(M.ref_dec*DTR),
		M.ref_dec + E->s.c.y / 3600 );		
	fclose(dbg);	
#endif

	interieur(&E->i,&I.i);        //return the triangle I.i inside the triangle E.i 
	e_transform(&I.i,dlsds,&I.s); //return the triangle I.i in the source plane
	e_inthere(E,&I,P,dlsds,A);	//return the small couple of triangles A in which P is located

	det=determinant(&A->s.a,&A->s.b,&A->s.c); //det is the surface of the source triangle
	if( fabs(det) < prec || it++ > nitermax )
		return;
	else
	{
		I.i = A->i;
		I.s = A->s;
		e_im_prec(&I,P,dlsds,prec,nitermax,A);
	}
}
