#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>
#include "lt.h"

/****************************************************************/
/*		nom:		e-pixel				*/
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			*/
/****************************************************************
 * Create a FITS file containing the computed images of the arcs 
 * and arclets in the source plane
 */
void	e_pixel(int np,char *name,struct galaxie *source)
{
	extern struct g_mode    M;
	//extern	struct	g_large	L;
	extern struct g_frame   F;
	extern struct g_source  S;
	extern struct g_observ	O;
	
	int	j,k,l;
	int	nx,ny,ny_sav;
	double scale;
	double xmin,ymin,xmax,ymax,dx,dy;
	double **zz;
	double f;
	struct point ps,pi;
	
	NPRINTF(stderr,"COMP: image of arcs and arclets --> %s\n",name);
	
	xmin=F.xmin;
	xmax=F.xmax;
	ymin=F.ymin;
	ymax=F.ymax;
	
	dx=xmax-xmin;
	dy=ymax-ymin;
	nx=ny=np;
	scale=Max(dx/nx,dy/ny);
	f=dx/scale; nx = f;  // BUG in C
	f=dy/scale; ny = f;  
	if(O.setbin)
	{
		if ( O.bin * nx <= NTMAX && O.bin * ny <= NTMAX )
		{
			nx=O.bin*nx;
			ny=O.bin*ny;
			scale=scale/((double) O.bin);
		}
		else
		{
			fprintf(stderr,"ERROR: reaching maximal size of array\n");
			exit(-1);
		}
	}

	dx=(nx-1)*scale;
	dy=(ny-1)*scale;
	xmax=xmin+dx;
	ymax=ymin+dy;
	
	NPRINTF(stderr,"\timage (%d,%d) s=%.3lf [%.3lf,%.3lf] [%.3lf,%.3lf]\n",
		nx,ny,scale,xmin,xmax,ymin,ymax);
	
	zz=(double **) alloc_square_double(ny,nx);
	// Set a source as point-like or extended
	// and assign it an intrinsic flux
	for( l = 0 ; l < S.ns ; l++ )
	{
		if ( source[l].E.a == 0. || source[l].E.b == 0. )
			source[l].c='s';
		else
			source[l].c='g';
			
		if ( source[l].I0 == 0. )
			source[l].I0=50.;			
	}	
	
	for(j=0;j<ny;j++)
	{
		NPRINTF(stderr, "...Compute line %d/%d\r",j+1,ny);
		pi.y=ymin+((double)j+1)*scale;
	
		for(k=0;k<nx;k++)
		{
			pi.x=xmin+((double)k+1)*scale;
			zz[j][k]=0.;
	
			for( l = 0 ; l < S.ns ; l++ )
			{
				e_dpl(&pi,source[l].dr,&ps);
				f=d_profil(ps.x,ps.y,&source[l]);
		/*                if ((f>sqrt(O.SKY/O.gain)) || (f>source[l].I0/100.))
		                 	zz[j][k]+=d_integrer(source[l],pi.x,pi.y,scale/2.,f,0);
				else
		*/
				zz[j][k]+=f;
			}
		}
	}
	// Delete the status line
	NPRINTF(stderr, "                             \r");
	
	if(O.setseeing)
		d_seeing(zz,nx,ny,scale);
	
	ny_sav = ny;
	if(O.setbin)
		d_binning(zz,&nx,&ny,O.bin);
	
	if(O.bruit)
		d_bruiter(zz,nx,ny);
	
	if( M.iref > 0 )
		wrf_fits_abs(name,zz,nx,ny,xmin,xmax,ymin,ymax,M.ref_ra,M.ref_dec);
	else
		wrf_fits(name,zz,nx,ny,xmin,xmax,ymin,ymax);
	
	free_square_double(zz,ny_sav);
}
