#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>

/****************************************************************/
/*		nom:		f_shape				*/
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			*/
/* lecture de fichiers ellipses, source ou image		*/
/****************************************************************/


static void getRADEC( char *line, 
                      int *iref, double *ra, double *dec );

static void convertXY( double *x, double *y, 
                       int iref, double ra, double dec );


void	f_shape( int *istart,
               struct galaxie *liste,
               char *name )
{
	extern  struct  g_mode          M;
	FILE	*IN;
	int	i,k;
	char	line[128];
	
	int		iref;
	double  ra, dec;
	
	i=(*istart);
	k=0;

	// default values
	ra = dec = 0;
	
	// Read the input arclet file
	NPRINTF(stderr,"READ: %s\n",name);		
	IN=fopen(name,"r");

	if( IN==NULL || ferror(IN) )
	{
		fprintf( stderr, "ERROR: Error reading %s\n", name);
		exit(-1);
	}
	
	while( fgets(line, 128, IN) != NULL && !feof(IN) && !ferror(IN) )
	{
		if( strstr(line, "#REFERENCE" ) != NULL )
		{
			NPRINTF( stderr, "%s", line );
			getRADEC( line, &iref, &ra, &dec );
		} 
		else if( sscanf(line,"%s%lf%lf%lf%lf%lf%lf%lf",
			liste[i].n,&liste[i].C.x,&liste[i].C.y,
			&liste[i].E.a,&liste[i].E.b,
			&liste[i].E.theta,&liste[i].z,&liste[i].mag) == 8 )
		{
			if( liste[i].n[0] != '#' ) 
			{
				// convert input to absolute coordinates
				convertXY( &liste[i].C.x, &liste[i].C.y, iref, ra, dec);
				
				// convert to output relative coordinates
				if( M.iref == 1 || M.iref == 3 )
				{
					// correction if M.ref_ra = 0
					if( fabs(liste[i].C.x - M.ref_ra) > 1 )
						liste[i].C.x -= M.ref_ra + 360.;	
					else
						liste[i].C.x -=M.ref_ra;
					liste[i].C.x *=-3600*cos(M.ref_dec*DTR);
					liste[i].C.y -=M.ref_dec;
					liste[i].C.y *=3600;
				} else if( M.iref == 2 )
				{
					liste[i].C.x -= M.ref_ra;
					liste[i].C.x -= M.ref_dec;			
				}
		
				liste[i].E.theta*=DTR;
				if ((liste[i].E.a==0.)||(liste[i].E.b==0.))
					liste[i].c='s';
				else
					liste[i].c='g';		
				
				// Initialise grad(2) variables
				liste[i].grad2.a = liste[i].grad2.c = 0;
				liste[i].grad.x = liste[i].grad.y = 0;
				
				i++;k++;
			}
		}
	}	

	
	fclose(IN);

	(*istart)=i;
	
	
	NPRINTF(stderr,"%d arclets read\n",i);
}

/* Return the ra and dec values in degrees and scaling factors
 * parameters :
 * - line : the #REFERENCE line
 */
static void getRADEC(char *line, 
			  int *iref, double *ra, double *dec )
{
	double	ss0,tt0;
	int		hh0,mm0,dd0,nn0;	
	
	char	ref1[20],ref2[20];	
	
	*ra = *dec = 0;
		
	if( sscanf(line, "%*s%d%s%s", 
			iref, ref1, ref2) == 3 )
	{
		// Convert ra and dec strings to double
		if( *iref == 1 )
		{
			sscanf(ref1,"%d:%d:%lf",&hh0,&mm0,&ss0);
			sscanf(ref2,"%d:%d:%lf",&dd0,&nn0,&tt0);

			*ra=((double)(hh0)+((double)(mm0))/60.+ss0/3600.)*15.;
		
			if (dd0<0)
				*dec=((double)(dd0))-((double)(nn0))/60-tt0/3600;
			else
				*dec=((double)(dd0))+((double)(nn0))/60+tt0/3600;
		}
		else if( *iref == 3 || *iref == 2 )
		{
			sscanf(ref1,"%lf",ra);
			sscanf(ref2,"%lf",dec);
		}
	}
}

/* Convert a relative coordinates to absolute coordinates
 */
static void convertXY( double *x, double *y, 
 						int iref, double ra, double dec )
{
	// Convert the input values to absolute WCS coordinates
	if( iref == 1 || iref == 3 )
	{
		*x /=-3600.*cos(dec*DTR);
		*x += ra;
		*y /=3600.;
		*y +=dec;
	} 
	else if( iref == 2 )  // image coordinates
	{
		*x += ra;
		*y += dec;
	}
}
