#include<stdio.h>
#include<signal.h>
#include<string.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>
#include<lt.h>

/****************************************************************/ 
/*		nom:		g_mass				*/ 
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			*/
/****************************************************************/

typedef void (*sighandler_t)(int);

static void computeKmap(int ny, int nx, double **map, double dlsds);
static double **nomass;	// not optimised clumps mass map for imass = 5
static int	isnomass;	// boolean 1 if nomap is computed 0 otherwise
static int  always; // boolean 1 if the potfile or cosmo is optimised
static void signalReset();
static int	interrupt;	// Global variable

void	g_mass(int imass,int np,double z,char *file)
{
	extern struct g_mode	M;
	extern struct g_frame	F;
	extern struct g_cosmo	C;
	extern struct g_pot	P;
	extern struct pot		lens[];
	extern int cblock[NPAMAX];
	
	register int	i,j;
	double	dx,dy;
	double	conv;
	double	dcrit,dcritA,dlsds,dl;
	double	**mass;		// mass map
	double	**meanmap, **stddevmap;	// used to compute the error mass map
	double	**array;	// bayes models
	int		nParam, nVal;	// size of array
	int		iVal;		// iterator over the array models (error mass map)
	int		nx, ny;		// image size
	char		mmfile[15];
	
	// Define the image size
	nx = np;
	ny = nx/(F.xmax-F.xmin)*(F.ymax-F.ymin);
	dx=(F.xmax-F.xmin)/(nx-1);
	dy=(F.ymax-F.ymin)/(ny-1);
	
	dlsds=dratio(lens[0].z,z);
	dl=distcosmo1(lens[0].z);
	dcrit=cH4piG*C.h/dl/dlsds;  // in g/cm^2 
	dcritA=cH0_4piG*C.h/dl/dlsds;  // in 10^12 M_sol/kpc^2
	conv=MCRIT12/C.h*dx*dy/dlsds*dl;  // in  10^12 M_sol/pixel
	
	if (imass==1)
	{
		NPRINTF(stderr,
			"COMP: projected relative mass map for z_s=%.3lf =>%s\n",z,file);
	}
	else if (imass==2)
	{
		NPRINTF(stderr,
			"COMP: projected absolute mass map in g/cm2 =>%s\n",file);
	}
	else if( imass==3)
	{
		NPRINTF(stderr,
			"COMP: integrated absolute mass map in 10^12 Msol/pixel =>%s\n",file);
	}
	else if (imass==4)
	{
		NPRINTF(stderr,
			"COMP: projected absolute mass map in 10^12 Msol/kpc2 =>%s\n",file);
	}
	else if( imass==5)
	{
		fprintf(stderr,
			"COMP: projected absolute error mass map in 10^12 Msol/pixel =>%s\n",file);
	}
	else
	{
		NPRINTF(stderr,
			"ERROR: mass keyword modifier %d in runmode section unknown\n",imass);
		return;
	} 
	
	NPRINTF(stderr,"dx:%lf dy:%lf conv:%lf dl:%lf dlsds:%lf\n",dx,dy,conv,dl,dlsds);
	
	// Initialise the mass maps(ny,nx)
	isnomass = 0;
	nomass = (double **) alloc_square_double(ny,nx);
	mass = (double **) alloc_square_double(ny,nx);
	
	// Compute the error mass map
	if( imass==5 )
	{
		array = readBayesModels( &nParam, &nVal );
		always = P.ircut || P.isigma || 
		         cblock[OMEGAM] || cblock[OMEGAX] || cblock[WX];
		
		if( nVal == 0 )
		{
			fprintf(stderr, "WARNING: bayes.dat not found.\n");
			return;
		}
		
		// initialize the mass map for imass = 5
		meanmap = (double **) alloc_square_double(ny,nx);
		stddevmap = (double **) alloc_square_double(ny,nx);
		
		// Initialize the interruption
		signal(SIGINT, signalReset);
		interrupt = 0;
		
		// For each line of <array> 		
		for( iVal = 0; iVal < nVal && !interrupt; iVal++ )
		{
			// set the lens parameters from <array>
			setBayesModel(iVal, nParam, nVal, array);		
			
			fprintf( stderr, "INFO: Compute the mass map %d/%d [CTRL-C to interrupt]\r", 
				iVal+1, nVal );					
			// Compute the convergence map
			computeKmap(ny,nx, mass, dlsds);
			
			// Store the value of each pixel in the mean and stddev maps
			for( j = 0; j < ny; j++ )
				for( i = 0; i < nx; i++ )					
				{
					meanmap[j][i] += mass[j][i];
					stddevmap[j][i] += mass[j][i]*mass[j][i];
				}
		}
		
		nVal = iVal; 	// in the case there is no interruption iVal == nVal
		
		fprintf( stderr, "\nINFO : Compute the error mass map\n" );
		// Compute the bias corrected standard deviation value of each pixel
		for( j = 0; j < ny; j++ )
			for( i = 0; i < nx; i++ )
			{
				stddevmap[j][i] /= nVal - 1;
				stddevmap[j][i] -= meanmap[j][i]*meanmap[j][i]/nVal/(nVal-1);
				meanmap[j][i] *= conv/nVal;
				mass[j][i] = sqrt( stddevmap[j][i] );
				mass[j][i] *= conv;	// convert to Msol/pixel
			}
		
		// write the mean mass map
		strcpy(mmfile, "meanmass.fits" );
		if (M.iref >0)
			wrf_fits_abs(mmfile,meanmap,nx,ny,F.xmin,F.xmax,F.ymin,F.ymax,M.ref_ra,M.ref_dec);
		else
			wrf_fits(mmfile,meanmap,nx,ny,F.xmin,F.xmax,F.ymin,F.ymax);
		

		// set the lens parameters from <array> back to the median value
		o_set_lens_bayes();
		
	} 
	else
	// when imass != 5
	{
		// Compute the convergence map
		computeKmap(ny,nx, mass, dlsds);
		
		for(j=0;j<ny;j++)
			for(i=0;i<nx;i++)
			{
				if (imass==2)
					mass[j][i]*=dcrit;
				else if (imass==3)
					mass[j][i]*=conv;
				else if (imass==4)
					mass[j][i]*=dcritA;
			};
	}


	/*
	size[0]=np;
	size[1]=np;
	wr_ipx(file,mass,2,size,"double","bin","real","projected_density",
			F.xmin,F.xmax,F.ymin,F.ymax);
	*/
	
	if (M.iref >0)
		wrf_fits_abs(file,mass,nx,ny,F.xmin,F.xmax,F.ymin,F.ymax,M.ref_ra,M.ref_dec);
	else
		wrf_fits(file,mass,nx,ny,F.xmin,F.xmax,F.ymin,F.ymax);
	
	free_square_double(mass,ny);
	if( imass == 5 )
	{
		free_square_double(meanmap,ny);
		free_square_double(stddevmap,ny);
	}
}



static void unmag_pot(struct point *pi, int ilmin, int ilmax, double dlsds, struct ellipse *eli)
{
	struct matrix grad2, grad2i;
	double a, b, c, delta, e, lambda, mu;
	int i;
	
	grad2.a = grad2.b = grad2.c = 0;
	
	for( i = ilmin; i < ilmax; i++)
	{
		grad2i = e_grad2_pot( pi, i );
		grad2.a += grad2i.a;
		grad2.b += grad2i.b;
		grad2.c += grad2i.c;
	}
		
	grad2.a *= dlsds;
	grad2.b *= dlsds;
	grad2.c *= dlsds;
	a = 1 - grad2.a;
	b = -grad2.b;
	c = 1 - grad2.c;


	// formeli()
	// eq carateristique : det(M-xI) = 0
	delta=(a-c)*(a-c)+4*b*b;	//  4*gamma^2 (cf phd_JPK eq 2.56)
	e=sqrt(delta);	/*e is 2 * shear, ie 2*gamma*/
	lambda=.5*(a+c+e);	// 1 - k + gamma
	mu=.5*(a+c-e);		// 1 - k - gamma

	eli->a=lambda;
	eli->b=mu;
	if(lambda!=mu && b!=0.)
		eli->theta=atan2(lambda-a,b); // cf phd_JPK eq 2.58
	else
		eli->theta=0.;
		
} 

/* Return a convergence map of size npxnp in pixels
 */
static void computeKmap(int ny, int nx, double **map, double dlsds)
{
	extern	struct	g_frame		F;
	extern	struct	g_grille	G;
	struct	point	pi;
	struct	ellipse	ampli;
	double	dx,dy;		// sampling size in arcsec/pixel
	register int	i,j;
	
	dx=(F.xmax-F.xmin)/(nx-1);
	dy=(F.ymax-F.ymin)/(ny-1);
	
	// Compute a k map for the not optimised clumps
	if( !isnomass || always )
	{
		for(j=0;j<ny;j++)
		{
			pi.y=j*dy+F.ymin;
			for(i=0;i<nx;i++)
			{
				pi.x=i*dx+F.xmin;
				unmag_pot(&pi, G.no_lens, G.nlens, dlsds, &ampli);
				nomass[j][i]=1.-(ampli.a+ampli.b)/2.; // ie map[j][i]=k
			}
		}
		isnomass = 1;
	}

	// Compute a k map for the optimised clumps
	for(j=0;j<ny;j++)
	{
		pi.y=j*dy+F.ymin;
		for(i=0;i<nx;i++)
		{
			pi.x=i*dx+F.xmin;
			unmag_pot(&pi, 0, G.no_lens, dlsds, &ampli);
			map[j][i]=nomass[j][i];
			map[j][i]+=1.-(ampli.a+ampli.b)/2.; // ie map[j][i]=k
		}
	}
	
}

static void signalReset(int param)
{
	interrupt = 1;
	signal(SIGINT, SIG_DFL);
}
