#include<stdio.h>
#include<signal.h>
#include<stdlib.h>
#include "dimension.h"
#include "structure.h"
#include "constant.h" 
#include "fonction.h"
#include "bayesChires.h"

/******************************************************************************
 * Name: bayes_chires.c
 * Authors: EJ
 * Date: 31/08/07
 * 
 * Analyse a bayes.dat file and compute a chires.dat file for each line. 
 * Finally, it produces a mean chires.dat file from all individual chires.dat 
 * file.
 *
 * syntax : bayes_chires <.par> 
 *
 * The bayes.dat file in the current directory is used.
 * Output : bayes_chires.dat
 *
 ******************************************************************************/
typedef void (*sighandler_t)(int);
static void signalReset();
int optInterrupt;

int main( int argc, char** argv )
{
	double **array; // contains the bayes.dat data
	int nParam, nVal;  // size of array
	int iVal;
	char fname[20]; // chires<ival>.dat
	double tmp;

	// Check the arguments
	if( argc < 2 || 
	    strstr( argv[1], ".par" ) == NULL )
	{
		fprintf(stderr, "Syntax : bayes_chires <.par>  [bayes.dat]\n");
		return -1;
	}

	// Read the .par file
	init_grille( argv[1], 1);
	
	// Read the image file
	readConstraints();

	// Read the bayes.dat file
	array = readBayesModels(&nParam, &nVal);
	if( array == NULL )
	{
		fprintf(stderr, "ERROR: bayes.dat file not found\n");
		return -1; 
	}

	// Handle CTRL-C to interrupt the optimisation but not lenstool
	signal(SIGINT, signalReset);
	optInterrupt = 0;

	// Loop over each line
	for( iVal = 0; iVal < nVal && !optInterrupt; iVal++ )
	{
		// Set the lens parameters from <array>
		setBayesModel( iVal, nParam, nVal, array );
		printf( "INFO: Compute the chires.dat file %d/%d  [CTRL-C to interrupt]\r", iVal+1, nVal);
		fflush(stdout);

		// Execute the chires()  --> chires<iVal>.dat
		sprintf( fname, "chires%d.dat", iVal );
		o_chires(fname);
	}
	printf( "\n" );
	free( array );
	return 0;
}


static void signalReset()
{
	signal(SIGINT, SIG_DFL);
	optInterrupt = 1;
	printf( "\nINFO: Optimisation interrupted by CTRL-C\r");
}
