#include<stdio.h>
#include<signal.h>
#include<stdlib.h>
#include "dimension.h"
#include "structure.h"
#include "constant.h" 
#include "fonction.h"
#include "bayesChires.h"

/******************************************************************************
 * Name: bayesImage.c
 * Authors: EJ
 * Date: 12/09/07
 * 
 * Analyse a bayes.dat file and compute a image.all file for each line. 
 *
 * syntax : bayesImage <.par> 
 *
 * The bayes.dat file in the current directory is used.
 * Output : image??.all
 *
 ******************************************************************************/
typedef void (*sighandler_t)(int);
static void signalReset();
int optInterrupt;

int main( int argc, char** argv )
{
	double **array; // contains the bayes.dat data
	int nParam, nVal;  // size of array
	int iVal;
	char fname[20]; // image<ival>.all
	int ni; // number of predicted images in image array
	int na; // number of image in the image catalog
	int i,j;

	// Check the arguments
	if( argc < 2 || 
	    strstr( argv[1], ".par" ) == NULL )
	{
		fprintf(stderr, "Syntax : bayesImage <.par>\n");
		return -1;
	}

	// Read the .par file
	init_grille( argv[1], 1);
	
	// Read catalog of multiple images 
	na = 0;
	f_shape( &na, arclet, I.multfile );

	// Initialise the grid
	if( G.pol != 0 )
		gridp();
	else
		grid();
	
	// Switch to silent mode
	M.verbose = 0;
	M.image = 1; // to cheat the e_lensing print out

	// Read the bayes.dat file
	array = readBayesModels(&nParam, &nVal);
	if( array == NULL )
	{
		fprintf(stderr, "ERROR: bayes.dat file not found\n");
		return -1; 
	}

	// Handle CTRL-C to interrupt the optimisation but not lenstool
	signal(SIGINT, signalReset);
	optInterrupt = 0;

	// Loop over each line
	for( iVal = 0; iVal < nVal && !optInterrupt; iVal++ )
	{
		// Set the lens parameters from <array>
		setBayesModel( iVal, nParam, nVal, array );
		printf( "INFO: Compute the image.all file %d/%d  [CTRL-C to interrupt]\r", iVal+1, nVal);
		fflush(stdout);

		// Reset the arclet grad(2) variables
		for( i = 0 ; i < na ; i++ )
		{
			arclet[i].grad.x=arclet[i].grad.y=0.;
			arclet[i].grad2.a=arclet[i].grad2.c=0.;
		}

		// arclet --> source list
		S.ns = 0; e_unlens(na, arclet, &S.ns, source);
		// source list --> image array
		e_lensing(source,image);

		// Count the number of images
		for( i = 0 ; i < S.ns ; i++ )
			for( j = 0 ; j < NIMAX && strcmp(image[i][j].n,"") ; j++ )
				cimage[ni++]=image[i][j];

		// write image array
		sprintf( fname, "image%d.all", iVal );
		ecrire_r(0,ni,cimage,fname,0);
	}
	printf( "\n" );
	free( array );
	return 0;
}


static void signalReset()
{
	signal(SIGINT, SIG_DFL);
	optInterrupt = 1;
	printf( "\nINFO: Optimisation interrupted by CTRL-C\r");
}
